=== added directory 'plugin/filtered_replicator'
=== added file 'plugin/filtered_replicator/AUTHORS'
--- plugin/filtered_replicator/AUTHORS	1970-01-01 00:00:00 +0000
+++ plugin/filtered_replicator/AUTHORS	2009-07-22 23:26:26 +0000
@@ -0,0 +1,2 @@
+Padraig O'Sullivan <osullivan.padraig@gmail.com>
+

=== added file 'plugin/filtered_replicator/filtered_replicator.cc'
--- plugin/filtered_replicator/filtered_replicator.cc	1970-01-01 00:00:00 +0000
+++ plugin/filtered_replicator/filtered_replicator.cc	2009-07-22 23:26:26 +0000
@@ -0,0 +1,207 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+/**
+ * @file
+ *
+ * Defines the implementation of a simple replicator that can filter
+ * events based on a schema or table name.
+ *
+ * @details
+ *
+ * This is a very simple implementation.  All we do is maintain two
+ * std::vectors:
+ *
+ *  1) contains all the schema names to filter
+ *  2) contains all the table names to filter
+ *
+ * If an event is on a schema or table in the vectors described above, then
+ * the event will not be passed along to the applier.
+ */
+
+#include "filtered_replicator.h"
+
+#include <drizzled/gettext.h>
+#include <drizzled/message/transaction.pb.h>
+
+#include <vector>
+#include <string>
+
+using namespace std;
+
+static bool sysvar_filtered_replicator_enabled= false;
+static char *sysvar_filtered_replicator_sch_filters= NULL;
+static char *sysvar_filtered_replicator_tab_filters= NULL;
+
+FilteredReplicator::FilteredReplicator(const char *in_sch_filters,
+                                       const char *in_tab_filters)
+{
+  /* 
+   * Add each of the specified schemas to the vector of schemas
+   * to filter.
+   */
+  if (in_sch_filters)
+  {
+    populateFilter(in_sch_filters, schemas_to_filter);
+  }
+
+  /* 
+   * Add each of the specified tables to the vector of tables
+   * to filter.
+   */
+  if (in_tab_filters)
+  {
+    populateFilter(in_tab_filters, tables_to_filter);
+  }
+}
+
+bool FilteredReplicator::isActive()
+{
+  return sysvar_filtered_replicator_enabled;
+}
+
+void FilteredReplicator::replicate(drizzled::plugin::Applier *in_applier, drizzled::message::Command *to_replicate)
+{
+  /* 
+   * We first check if this event should be filtered or not...
+   */
+  if (isSchemaFiltered(to_replicate->schema()) ||
+      isTableFiltered(to_replicate->table()))
+  {
+    return;
+  }
+
+  /*
+   * We can now simply call the applier's apply() method, passing
+   * along the supplied command.
+   */
+  in_applier->apply(to_replicate);
+}
+
+void FilteredReplicator::populateFilter(const char *input,
+                                        vector<string> &filter)
+{
+  string filter_list(input);
+  string::size_type last_pos= filter_list.find_first_not_of(',', 0);
+  string::size_type pos= filter_list.find_first_of(',', last_pos);
+
+  while (pos != string::npos || last_pos != string::npos)
+  {
+    filter.push_back(filter_list.substr(last_pos, pos - last_pos));
+    last_pos= filter_list.find_first_not_of(',', pos);
+    pos= filter_list.find_first_of(',', last_pos);
+  }
+}
+
+bool FilteredReplicator::isSchemaFiltered(const string &schema_name)
+{
+  vector<string>::iterator it= find(schemas_to_filter.begin(),
+                                    schemas_to_filter.end(),
+                                    schema_name);
+  if (it != schemas_to_filter.end())
+  {
+    return true;
+  }
+  return false;
+}
+
+bool FilteredReplicator::isTableFiltered(const string &table_name)
+{
+  vector<string>::iterator it= find(tables_to_filter.begin(),
+                                    tables_to_filter.end(),
+                                    table_name);
+  if (it != tables_to_filter.end())
+  {
+    return true;
+  }
+  return false;
+}
+
+static FilteredReplicator *filtered_replicator= NULL; /* The singleton replicator */
+
+static int init(PluginRegistry &registry)
+{
+  if (sysvar_filtered_replicator_enabled)
+  {
+    filtered_replicator= new(std::nothrow) 
+      FilteredReplicator(sysvar_filtered_replicator_sch_filters,
+                         sysvar_filtered_replicator_tab_filters);
+    if (filtered_replicator == NULL)
+    {
+      return 1;
+    }
+    registry.add(filtered_replicator);
+  }
+  return 0;
+}
+
+static int deinit(PluginRegistry &registry)
+{
+  if (filtered_replicator)
+  {
+    registry.remove(filtered_replicator);
+    delete filtered_replicator;
+  }
+  return 0;
+}
+
+static DRIZZLE_SYSVAR_BOOL(enable,
+                           sysvar_filtered_replicator_enabled,
+                           PLUGIN_VAR_NOCMDARG,
+                           N_("Enable filtered replicator"),
+                           NULL, /* check func */
+                           NULL, /* update func */
+                           false /*default */);
+static DRIZZLE_SYSVAR_STR(filteredschemas,
+                          sysvar_filtered_replicator_sch_filters,
+                          PLUGIN_VAR_OPCMDARG,
+                          N_("List of schemas to filter"),
+                          NULL,
+                          NULL,
+                          false);
+static DRIZZLE_SYSVAR_STR(filteredtables,
+                          sysvar_filtered_replicator_tab_filters,
+                          PLUGIN_VAR_OPCMDARG,
+                          N_("List of tables to filter"),
+                          NULL,
+                          NULL,
+                          false);
+
+static struct st_mysql_sys_var* filtered_replicator_system_variables[]= {
+  DRIZZLE_SYSVAR(enable),
+  DRIZZLE_SYSVAR(filteredschemas),
+  DRIZZLE_SYSVAR(filteredtables),
+  NULL
+};
+
+drizzle_declare_plugin(filtered_replicator)
+{
+  "filtered_replicator",
+  "0.1",
+  "Padraig O'Sullivan",
+  N_("Filtered Replicator"),
+  PLUGIN_LICENSE_GPL,
+  init, /* Plugin Init */
+  deinit, /* Plugin Deinit */
+  NULL, /* status variables */
+  filtered_replicator_system_variables, /* system variables */
+  NULL    /* config options */
+}
+drizzle_declare_plugin_end;

=== added file 'plugin/filtered_replicator/filtered_replicator.h'
--- plugin/filtered_replicator/filtered_replicator.h	1970-01-01 00:00:00 +0000
+++ plugin/filtered_replicator/filtered_replicator.h	2009-07-22 23:26:26 +0000
@@ -0,0 +1,109 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+/**
+ * @file
+ *
+ * Defines the API of a simple filtered replicator.
+ *
+ * @see drizzled/plugin/replicator.h
+ * @see drizzled/plugin/applier.h
+ */
+
+#ifndef DRIZZLE_PLUGIN_FILTERED_REPLICATOR_H
+#define DRIZZLE_PLUGIN_FILTERED_REPLICATOR_H
+
+#include <drizzled/server_includes.h>
+#include <drizzled/atomics.h>
+#include <drizzled/plugin/replicator.h>
+#include <drizzled/plugin/applier.h>
+
+#include <vector>
+#include <string>
+
+class FilteredReplicator: public drizzled::plugin::Replicator
+{
+public:
+  FilteredReplicator(const char *in_sch_filters,
+                     const char *in_tab_filters);
+
+  /** Destructor */
+  ~FilteredReplicator() {}
+
+  /**
+   * Replicate a Command message to an Applier.
+   *
+   * @note
+   *
+   * It is important to note that memory allocation for the 
+   * supplied pointer is not guaranteed after the completion 
+   * of this function -- meaning the caller can dispose of the
+   * supplied message.  Therefore, replicators and appliers 
+   * implementing an asynchronous replication system must copy
+   * the supplied message to their own controlled memory storage
+   * area.
+   *
+   * @param Command message to be replicated
+   */
+  void replicate(drizzled::plugin::Applier *in_applier, drizzled::message::Command *to_replicate);
+  
+  /** 
+   * Returns whether the replicator is active.
+   */
+  bool isActive();
+
+private:
+ 
+  /**
+   * Given a comma-separated string, parse that string to obtain
+   * each entry and add each entry to the supplied vector.
+   *
+   * @param[in] input a comma-separated string of entries
+   * @param[out] filter a std::vector to be populated with the entries
+   *                    from the input string
+   */
+  void populateFilter(const char *input,
+                      std::vector<std::string> &filter);
+
+  /**
+   * Search the vector of schemas to filter to determine whether
+   * the given schema should be filtered or not. The parameter
+   * is obtained from the Command message passed to the replicator.
+   *
+   * @param[in] schema_name name of schema to search for
+   * @return true if the given schema should be filtered; false otherwise
+   */
+  bool isSchemaFiltered(const std::string &schema_name);
+
+  /**
+   * Search the vector of tables to filter to determine whether
+   * the given table should be filtered or not. The parameter
+   * is obtained from the Command message passed to the replicator.
+   *
+   * @param[in] table_name name of table to search for
+   * @return true if the given table should be filtered; false otherwise
+   */
+  bool isTableFiltered(const std::string &table_name);
+
+  std::vector<std::string> schemas_to_filter;
+  std::vector<std::string> tables_to_filter;
+};
+
+#endif /* DRIZZLE_PLUGIN_FILTERED_REPLICATOR_H */

=== added file 'plugin/filtered_replicator/plugin.ini'
--- plugin/filtered_replicator/plugin.ini	1970-01-01 00:00:00 +0000
+++ plugin/filtered_replicator/plugin.ini	2009-07-22 23:26:26 +0000
@@ -0,0 +1,8 @@
+[plugin]
+name=filtered_replicator
+title=Filtered Replicator
+description=A simple filtered replicator which allows a user to filter out
+            events based on a schema or table name
+load_by_default=yes
+sources=filtered_replicator.cc
+headers=filtered_replicator.h

